package com.zhentao.config;

import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.config.Config;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

// 定义这是一个Spring配置类，用于配置Redisson相关的Bean
@Configuration
public class RedissonConfig {
//     从配置文件（如application.properties或application.yml）中获取Redis服务器的主机地址
    @Value("${spring.redis.host}")
    private String host;
    // 从配置文件中获取Redis服务器的端口号
    @Value("${spring.redis.port}")
    private int port;
    // 从配置文件中获取要使用的Redis数据库索引
    @Value("${spring.redis.database}")
    private int database;
    // 定义一个Bean，名称为getRedisson，返回一个RedissonClient实例，Spring会在需要RedissonClient的地方注入这个实例
    @Bean
    public RedissonClient getRedisson(){

        // 创建一个Redisson的配置对象z
        Config config = new Config();

        // 配置Redisson使用单机模式，并设置Redis服务器的地址
        config.useSingleServer().setAddress("redis://"+host+":"+port)

                // 设置要使用的Redis数据库索引
                .setDatabase(database)
                ;


        // 设置锁的看门狗超时时间为2000毫秒。看门狗用于自动延长锁的持有时间，防止业务执行时间过长导致锁提前释放
        config.setLockWatchdogTimeout(2000);

        // 根据配置创建并返回RedissonClient实例
        return Redisson.create(config);
    }
}
