package com.zhentao.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.domain.UserLogin;
import com.zhentao.dto.user.*;
import com.zhentao.dto.Result;
import com.zhentao.enums.ApiServerException;
import com.zhentao.exception.AsynException;
import com.zhentao.mapper.UserLoginMapper;
import com.zhentao.service.FileService;
import com.zhentao.service.UserLoginService;
import com.zhentao.tool.TokenUtils;
import io.minio.errors.*;
import jdk.nashorn.internal.runtime.regexp.joni.exception.InternalException;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.management.openmbean.InvalidKeyException;
import java.io.IOException;
import java.rmi.ServerException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
* @author 86183
* @description 针对表【user_login(用户)】的数据库操作Service实现
* @createDate 2025-05-04 21:01:51
*/
@Service
public class UserLoginServiceImpl extends ServiceImpl<UserLoginMapper, UserLogin>
    implements UserLoginService{
    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private UserLoginMapper userLoginMapper;

    @Autowired
    private FileService fileService;
    @Override
    public Result login(UserDto userDto) {
        RLock lock = redissonClient.getLock(userDto.getPhone()+"phone");
        try {
            boolean b = lock.tryLock(10, 20, TimeUnit.SECONDS);
            if (b){
                System.err.println(userDto);
                String s = stringRedisTemplate.opsForValue().get(userDto.getPhone());
                System.err.println(s);
//                验证码比较
                if (!s.equals(userDto.getCode())){
                    throw new AsynException(ApiServerException.NOTE_ERROR);
                }
                try{
                    QueryWrapper<UserLogin> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("user_mobile",userDto.getPhone());
                    UserLogin one = this.getOne(queryWrapper);
                    String jwtToken = TokenUtils.createJwtToken(one.getId() + "");
                    return Result.OK("登录成功",jwtToken);
                }catch (NullPointerException e){
                    long l = IdUtil.getSnowflake(1, 1).nextId();
                    UserLogin userLogin = new UserLogin();
                    userLogin.setId(l);
                    userLogin.setUserMobile(userDto.getPhone());
                    userLogin.setCreatedTime(new Date());

                    this.save(userLogin);
                    String jwtToken = TokenUtils.createJwtToken(l + "");
                    return Result.OK("登录成功",jwtToken);
                }
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new AsynException(ApiServerException.INTERRUPT);
        }finally {
            lock.unlock();
        }
        return null;
    }
    @Override
    public Result note(NoteDto noteDto) {
        int randomSixDigit = 100000 + (int) (Math.random() * 900000); // 生成六位随机数
        System.out.println("验证码："+randomSixDigit);
        System.err.println("手机号："+noteDto.getPhone());
        stringRedisTemplate.opsForValue().set(noteDto.getPhone(),randomSixDigit+"");

//        String host = "https://gyytz.market.alicloudapi.com";
//        String path = "/sms/smsSend";
//        String method = "POST";
//        String appcode = "03a7c868bfcb4de291d6d609a6b625d3";
//        Map<String, String> headers = new HashMap<String, String>();
//        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
//        headers.put("Authorization", "APPCODE " + appcode);
//        Map<String, String> querys = new HashMap<String, String>();
//        querys.put("mobile", noteDto.getPhone());
//        querys.put("param", "**code**:"+randomSixDigit+",**minute**:5");
//
////smsSignId（短信前缀）和templateId（短信模板），可登录国阳云控制台自助申请。参考文档：http://help.guoyangyun.com/Problem/Qm.html
//
//        querys.put("smsSignId", "2e65b1bb3d054466b82f0c9d125465e2");
//        querys.put("templateId", "908e94ccf08b4476ba6c876d13f084ad");
//        Map<String, String> bodys = new HashMap<String, String>();
//
//
//        try {
//            /**
//             * 重要提示如下:
//             * HttpUtils请从\r\n\t    \t* https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/src/main/java/com/aliyun/api/gateway/demo/util/HttpUtils.java\r\n\t    \t* 下载
//             *
//             * 相应的依赖请参照
//             * https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/pom.xml
//             */
//            HttpResponse response = (HttpResponse) HttpUtils.doPost(host, path, method, headers, querys, bodys);
//            System.out.println(response.toString());
//            //获取response的body
//            //System.out.println(EntityUtils.toString(response.getEntity()));
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
        return Result.OK("发送成功",null);
    }

    @Override
    public Result login_up(UserDtoup userDtoup) {
        RLock lock = redissonClient.getLock(userDtoup.username);
        try {
            boolean b = lock.tryLock(10, 20, TimeUnit.SECONDS);
            if (b){
                QueryWrapper<UserLogin> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("user_username",userDtoup.getUsername());
                UserLogin one = this.getOne(queryWrapper);
                if (one==null){
                    throw new AsynException(ApiServerException.NULL_USERNAME);
                }
                String salt = one.getSalt();
                String s = DigestUtils.md5DigestAsHex((salt + userDtoup.getPassword()).getBytes());
                if (!s.equals(one.getUserPassword())){
                    throw new AsynException(ApiServerException.NULL_PASSWORD);
                }
                String jwtToken = TokenUtils.createJwtToken(one.getId()+"");
                return Result.OK("登录成功",jwtToken);
            }else {
                return Result.ERR("获取锁超时",null);
            }
        }catch (InterruptedException e){
            Thread.currentThread().interrupt();
            return Result.ERR("线程被中断",null);
        }finally {
            lock.unlock();
        }
    }
//注册
    @Override
    public Result register(UserRegister userRegister) {
        System.err.println(userRegister);
        RLock lock = redissonClient.getLock(userRegister.getPhone()+userRegister.getUsername());
        try{
            boolean b = lock.tryLock(10, 20, TimeUnit.SECONDS);
            if (b){
//                用来判断验证码是否正确
                String s = stringRedisTemplate.opsForValue().get(userRegister.getPhone());
                System.err.println("redis取出来的验证码："+s);
                if (!s.equals(userRegister.getCode())){
                    throw new AsynException(ApiServerException.NOTE_ERROR);
                }
//                根据手机号查询信息
                QueryWrapper<UserLogin> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("user_mobile",userRegister.getPhone());
                UserLogin one = this.getOne(queryWrapper);
                if (one==null){
                    UserLogin userLogin = new UserLogin();
                    userLogin.setUserMobile(userRegister.getPhone());
                    userLogin.setUserUsername(userRegister.getUsername());
//                    随机字符串
                    String uuid = String.valueOf(UUID.randomUUID());
                    userLogin.setSalt(uuid);
//                    md5加密
                    String s1 = DigestUtils.md5DigestAsHex((uuid + userRegister.getPassword()).getBytes());
                    userLogin.setUserPassword(s1);
                    long l = IdUtil.getSnowflake(1, 1).nextId();
//                    进行注册
                    boolean save = this.save(userLogin);
                    if (save){
                        return Result.OK("注册成功",save);
                    }else {
                        return Result.ERR("注册失败",save);
                    }
                }else {
                    one.setUserUsername(userRegister.getUsername());
                    //                    随机字符串
                    String uuid = String.valueOf(UUID.randomUUID());
                    one.setSalt(uuid);
//                    md5加密
                    String s1 = DigestUtils.md5DigestAsHex((uuid + userRegister.getPassword()).getBytes());
                    one.setUserPassword(s1);
//                    进行修改
                    boolean b1 = this.updateById(one);
                    if (b1){
                        return Result.OK("注册成功",b1);
                    }else {
                        return Result.ERR("注册失败",b1);
                    }
                }
            }
        }catch (InterruptedException e){
            Thread.currentThread().interrupt();
        }finally {
            lock.unlock();
        }
        return null;
    }

    @Override
    public Result uploadPicture(MultipartFile multipartFile, Long id) throws IOException, ServerException, InvalidBucketNameException, InsufficientDataException, ErrorResponseException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException, io.minio.errors.ServerException, java.security.InvalidKeyException, io.minio.errors.InternalException {
        if (multipartFile == null) {
            Result.ERR("上传失败", null);
        }
        UserLogin userLogin = userLoginMapper.selectById(id);
        String url = fileService.uploadImage(multipartFile.getOriginalFilename(), multipartFile.getInputStream());
        System.err.println(url);
        userLogin.setAvatar(url);
        userLoginMapper.updateById(userLogin);
        return Result.OK("上传成功", url);
    }

    @Override
    public UserLogin getUserInfo(Long id) {
        return userLoginMapper.selectById(id);
    }

    @Override
    public Result updatePassword(String username, String oldPassword, String newPassword) {
        // 获取分布式锁，防止并发修改密码问题
        RLock lock = redissonClient.getLock(username);
        try {
            // 尝试获取锁，等待10秒，锁自动释放时间为20秒
            boolean b = lock.tryLock(10, 20, java.util.concurrent.TimeUnit.SECONDS);
            if (b) {
                // 创建查询条件，根据用户名查询用户信息
                QueryWrapper<UserLogin> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("user_username", username);
                UserLogin one = this.getOne(queryWrapper);
                if (one == null) {
                    // 如果用户不存在，抛出异常
                    throw new AsynException(ApiServerException.NULL_USERNAME);
                }
                String salt = one.getSalt();
                // 使用之前相同的加密逻辑，对旧密码进行加密
                String oldPasswordEncrypted = DigestUtils.md5DigestAsHex((salt + oldPassword).getBytes());
                if (!oldPasswordEncrypted.equals(one.getUserPassword())) {
                    // 如果旧密码加密后与数据库中存储的密码不匹配，抛出异常
                    throw new AsynException(ApiServerException.NULL_PASSWORD);
                }
                // 使用相同的加密逻辑，对新密码进行加密
                String newPasswordEncrypted = DigestUtils.md5DigestAsHex((salt + newPassword).getBytes());
                // 更新用户密码到数据库
                one.setUserPassword(newPasswordEncrypted);
                boolean updateResult = this.updateById(one);
                if (updateResult) {
                    return Result.OK("修改密码成功", updateResult);
                } else {
                    return Result.ERR("修改密码失败", null);
                }
            } else {
                return Result.ERR("获取锁超时",null);
            }
        } catch (Exception e) {
            // 捕获可能的异常，如获取锁时中断异常等
            if (e instanceof InterruptedException) {
                Thread.currentThread().interrupt();
            }
            String message = e.getMessage();
            return Result.ERR("修改密码失败：" + message,null);
        } finally {
            // 无论是否成功获取锁或修改密码，都要释放锁
            if (lock.isLocked() && lock.isHeldByCurrentThread()) {
                lock.unlock();
            }
        }
    }

    @Override
    public boolean updateUserInfo(UserInfoDto userInfoDto, Long userId) {
        // 根据userId查询用户原始信息
        QueryWrapper<UserLogin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", userId);
        UserLogin userLogin = userLoginMapper.selectOne(queryWrapper);
        if (userLogin == null) {
            // 用户不存在，返回修改失败
            return false;
        }
        // 更新头像图片字段
        if (userInfoDto.getAvator()!= null) {
            userLogin.setAvatar(userInfoDto.getAvator());
        }
        // 更新用户昵称字段
        if (userInfoDto.getUserName() != null) {
            userLogin.setUserName(userInfoDto.getUserName());
        }
        // 更新手机号字段
        if (userInfoDto.getUserMobile() != null) {
            userLogin.setUserMobile(userInfoDto.getUserMobile());
        }
        // 将更新后的用户信息更新到数据库
        int updateResult = userLoginMapper.updateById(userLogin);
        return updateResult > 0;
    }
}




